package istia.st.mvc.personnes.tests;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Iterator;
import org.springframework.beans.factory.xml.XmlBeanFactory;
import org.springframework.core.io.ClassPathResource;

import istia.st.mvc.personnes.dao.DaoException;
import istia.st.mvc.personnes.service.IService;
import istia.st.mvc.personnes.entites.Personne;
import junit.framework.TestCase;

public class TestServiceFirebird extends TestCase {

	// couche [service]
	private IService service;

	public IService getService() {
		return service;
	}

	public void setService(IService service) {
		this.service = service;
	}

	// constructeur
	public void setUp() {
		service = (IService) (new XmlBeanFactory(new ClassPathResource(
				"spring-config-test-service-firebird.xml"))).getBean("service");
	}

	// liste des personnes
	private void doListe(Collection personnes) {
		Iterator iter = personnes.iterator();
		while (iter.hasNext()) {
			System.out.println(iter.next());
		}
	}

	// test1
	public void test1() throws ParseException {
		// liste actuelle
		Collection personnes = service.getAll();
		int nbPersonnes = personnes.size();
		// affichage
		doListe(personnes);
		// ajout d'une personne
		Personne p1 = new Personne(-1, "X", "X", new SimpleDateFormat(
				"dd/MM/yyyy").parse("01/02/2006"), true, 1);
		service.saveOne(p1);
		int id1 = p1.getId();
		// vrification - on aura un plantage si la personne n'est pas trouve
		p1 = service.getOne(id1);
		assertEquals("X", p1.getNom());
		// modification
		p1.setNom("Y");
		service.saveOne(p1);
		// vrification - on aura un plantage si la personne n'est pas trouve
		p1 = service.getOne(id1);
		assertEquals("Y", p1.getNom());
		// suppression
		service.deleteOne(id1);
		// vrification
		int codeErreur = 0;
		boolean erreur = false;
		try {
			p1 = service.getOne(id1);
		} catch (DaoException ex) {
			erreur = true;
			codeErreur = ex.getCode();
		}
		// on doit avoir une erreur de code 2
		assertTrue(erreur);
		assertEquals(2, codeErreur);
		// liste des personnes
		personnes = service.getAll();
		assertEquals(nbPersonnes, personnes.size());
	}

	// modification-suppression d'un lment inexistant
	public void test2() throws ParseException {
		// d'abord ajout
		Personne p1 = new Personne(-1, "X", "X", new SimpleDateFormat(
				"dd/MM/yyyy").parse("01/02/2006"), true, 1);
		service.saveOne(p1);
		// rcupration copie de la personne
		Personne p2 = service.getOne(p1.getId());
		// modification copie avec un id inexistant
		p2.setId(p1.getId() + 1);
		// modification nom de la copie
		p2.setNom("Y");
		// sauvegarde copie - on doit avoir une DaoException de code 2
		boolean erreur = false;
		int codeErreur = 0;
		try {
			service.saveOne(p2);
		} catch (DaoException ex) {
			erreur = true;
			codeErreur = ex.getCode();
		}
		// vrification - on doit avoir une erreur de code 2
		assertTrue(erreur);
		assertEquals(2, codeErreur);
		// suppression lment inexistant - on doit avoir une DaoException de
		// code 2
		erreur = false;
		codeErreur = 0;
		try {
			service.deleteOne(-2);
		} catch (DaoException ex) {
			erreur = true;
			codeErreur = ex.getCode();
		}
		// vrification - on doit avoir une erreur de code 2
		assertTrue(erreur);
		assertEquals(2, codeErreur);
		// suppression personne p1
		service.deleteOne(p1.getId());
		// vrification
		try {
			p1 = service.getOne(p1.getId());
		} catch (DaoException ex) {
			erreur = true;
			codeErreur = ex.getCode();
		}
		// on doit avoir une erreur de code 2
		assertTrue(erreur);
		assertEquals(2, codeErreur);
	}

	// gestion des versions de personne
	public void test3() throws ParseException, InterruptedException {
		// d'abord ajout
		Personne p1 = new Personne(-1, "X", "X", new SimpleDateFormat(
				"dd/MM/yyyy").parse("01/02/2006"), true, 1);
		service.saveOne(p1);
		// rcupration copie p2 de la personne p1
		Personne p2 = service.getOne(p1.getId());
		// rcupration copie p3 de la personne p1
		Personne p3 = service.getOne(p1.getId());
		// attente 10 ms
		Thread.sleep(10);
		// sauvegarde copie p2 - la version de p1 va changer
		service.saveOne(p2);
		// sauvegarde copie p3 - on doit avoir une DaoException de code 2
		boolean erreur = false;
		int codeErreur = 0;
		try {
			service.saveOne(p3);
		} catch (DaoException ex) {
			erreur = true;
			codeErreur = ex.getCode();
		}
		// vrification - on doit avoir une erreur de code 2
		assertTrue(erreur);
		assertEquals(codeErreur, 2);
		// suppression personne p1
		service.deleteOne(p1.getId());
		// vrification
		try {
			p1 = service.getOne(p1.getId());
		} catch (DaoException ex) {
			erreur = true;
			codeErreur = ex.getCode();
		}
		// on doit avoir une erreur de code 2
		assertTrue(erreur);
		assertEquals(2, codeErreur);
	}

	// optimistic locking - accs multi-threads
	public void test4() throws Exception {
		// ajout d'une personne
		Personne p1 = new Personne(-1, "X", "X", new SimpleDateFormat(
				"dd/MM/yyyy").parse("01/02/2006"), true, 0);
		service.saveOne(p1);
		int id1 = p1.getId();
		// cration de N threads de mise  jour du nombre d'enfants
		final int N = 100;
		Thread[] taches = new Thread[N];
		for (int i = 0; i < taches.length; i++) {
			taches[i] = new ThreadServiceMajEnfants("thread n " + i, service,
					id1);
			taches[i].start();
		}
		// on attend la fin des threads
		for (int i = 0; i < taches.length; i++) {
			taches[i].join();
		}
		// on rcupre la personne
		p1 = service.getOne(id1);
		// elle doit avoir N enfants
		assertEquals(N, p1.getNbEnfants());
		// suppression personne p1
		service.deleteOne(p1.getId());
		// vrification
		boolean erreur = false;
		int codeErreur = 0;
		try {
			p1 = service.getOne(p1.getId());
		} catch (DaoException ex) {
			erreur = true;
			codeErreur = ex.getCode();
		}
		// on doit avoir une erreur de code 2
		assertTrue(erreur);
		assertEquals(2, codeErreur);
	}

	// tests de validit de saveOne
	public void test5() throws ParseException {
		int codeErreur = 0;
		// personne null
		try {
			service.saveOne(null);
		} catch (DaoException ex) {
			codeErreur = ex.getCode();
		}
		assertEquals(10, codeErreur);
		// id erron
		try {
			service.saveOne(new Personne(-2, "X", "X", new SimpleDateFormat(
					"dd/MM/yyyy").parse("01/02/2006"), true, 0));
		} catch (DaoException ex) {
			codeErreur = ex.getCode();
		}
		assertEquals(11, codeErreur);
		// date de naissance manquante
		try {
			service.saveOne(new Personne(100, "X", "X", null, true, 0));
		} catch (DaoException ex) {
			codeErreur = ex.getCode();
		}
		assertEquals(12, codeErreur);
		// nombre d'enfants invalide
		try {
			service.saveOne(new Personne(100, "X", "X", new SimpleDateFormat(
					"dd/MM/yyyy").parse("01/02/2006"), true, -2));
		} catch (DaoException ex) {
			codeErreur = ex.getCode();
		}
		assertEquals(13, codeErreur);
		// nom manquant
		try {
			service.saveOne(new Personne(100, "X", "    ",
					new SimpleDateFormat("dd/MM/yyyy").parse("01/02/2006"),
					true, 2));
		} catch (DaoException ex) {
			codeErreur = ex.getCode();
		}
		assertEquals(14, codeErreur);
		// prnom manquant
		try {
			service.saveOne(new Personne(100, "  ", "X", new SimpleDateFormat(
					"dd/MM/yyyy").parse("01/02/2006"), true, 2));
		} catch (DaoException ex) {
			codeErreur = ex.getCode();
		}
		assertEquals(15, codeErreur);
	}

	// tests de la mthode deleteMany
	public void test7() throws ParseException {
		// liste actuelle
		Collection personnes = service.getAll();
		int nbPersonnes1 = personnes.size();
		// affichage
		doListe(personnes);
		// cration de trois personnes
		Personne p1 = new Personne(-1, "X", "X", new SimpleDateFormat(
				"dd/MM/yyyy").parse("01/02/2006"), true, 1);
		Personne p2 = new Personne(-1, "Y", "Y", new SimpleDateFormat(
				"dd/MM/yyyy").parse("01/03/2006"), false, 0);
		Personne p3 = new Personne(-2, "Z", "Z", new SimpleDateFormat(
				"dd/MM/yyyy").parse("01/04/2006"), true, 2);
		// ajout des 3 personnes - la personne p3 avec l'id -2  va provoquer
		// une exception
		boolean erreur = false;
		try {
			service.saveMany(new Personne[] { p1, p2, p3 });
		} catch (Exception ex) {
			erreur = true;
			System.out.println(ex.toString());
		}
		// vrification
		assertTrue(erreur);
		// nouvelle liste - le nombre d'lments n'a pas du changer
		//  cause rollback automatique de la transaction
		int nbPersonnes2 = service.getAll().size();
		assertEquals(nbPersonnes1, nbPersonnes2);
		// ajout des deux personnes valides
		// on remet leur id  -1
		p1.setId(-1);
		p2.setId(-1);
		service.saveMany(new Personne[] { p1, p2 });
		// on rcupre leurs id
		int id1 = p1.getId();
		int id2 = p2.getId();
		// vrifications
		p1 = service.getOne(id1);
		assertEquals(p1.getNom(), "X");
		p2 = service.getOne(id2);
		assertEquals(p2.getNom(), "Y");
		// nouvelle liste - on doit avoir 2 lments de +
		int nbPersonnes3 = service.getAll().size();
		assertEquals(nbPersonnes1 + 2, nbPersonnes3);
		// suppression de p1 et p2 et d'une personne inexistante
		// une exception doit se produire
		erreur = false;
		try {
			service.deleteMany(new int[] { id1, id2, -1 });
		} catch (Exception ex) {
			erreur = true;
			System.out.println(ex.toString());
		}
		// vrification
		assertTrue(erreur);
		// nouvelle liste
		personnes = service.getAll();
		int nbPersonnes4 = personnes.size();
		// aucune personne n'a du tre supprime (rollback
		// automatique de la transaction)
		assertEquals(nbPersonnes4, nbPersonnes3);
		// on supprime les deux personnes valides
		service.deleteMany(new int[] { id1, id2 });
		// vrifications
		// personne p1
		erreur = false;
		int codeErreur = 0;
		try {
			p1 = service.getOne(id1);
		} catch (DaoException ex) {
			erreur = true;
			codeErreur = ex.getCode();
		}
		// on doit avoir une erreur de code 2
		assertTrue(erreur);
		assertEquals(2, codeErreur);
		// personne p2
		erreur = false;
		codeErreur = 0;
		try {
			p1 = service.getOne(id2);
		} catch (DaoException ex) {
			erreur = true;
			codeErreur = ex.getCode();
		}
		// on doit avoir une erreur de code 2
		assertTrue(erreur);
		assertEquals(2, codeErreur);
		// nouvelle liste
		personnes = service.getAll();
		int nbPersonnes5 = personnes.size();
		// vrification - on doit tre revenu au point de dpart
		assertEquals(nbPersonnes5, nbPersonnes1);
		// affichage
		doListe(personnes);
	}

	// insertions multi-threads
	public void test6() throws ParseException, InterruptedException{
		// cration d'une personne
		Personne p = new Personne(-1, "X", "X", new SimpleDateFormat(
				"dd/MM/yyyy").parse("01/02/2006"), true, 0);
		// qu'on duplique N fois dans un tableau
		final int N = 10;
		Personne[] personnes=new Personne[N];
		for(int i=0;i<personnes.length;i++){
			personnes[i]=new Personne(p);
		}
		// cration de N threads d'insertion - chaque thread insre 1 personne
		Thread[] taches = new Thread[N];
		for (int i = 0; i < taches.length; i++) {
			taches[i] = new ThreadServiceInsertPersonne("thread n " + i, service, personnes[i]);
			taches[i].start();
		}
		// on attend la fin des threads
		for (int i = 0; i < taches.length; i++) {
			// thread n i
			taches[i].join();
			// supression personne
			service.deleteOne(personnes[i].getId());
		}
	}

}